@@ -6,7 +6,7 @@
    It is dual-licensed under "the GNU LGPLv3+ or the GNU GPLv2+".
    You can redistribute it and/or modify it under either
      - the terms of the GNU Lesser General Public License as published
-       by the Free Software Foundation; either version 3, or (at your
+       by the Free Software Foundation, either version 3, or (at your
        option) any later version, or
      - the terms of the GNU General Public License as published by the
        Free Software Foundation; either version 2, or (at your option)
@@ -28,7 +28,6 @@
 /* Specification.  */
 #include "unigbrk.h"
 
-#include <stdbool.h>
 #include <string.h>
 
 #include "unictype.h"
